#include "Play.hpp"

#include <Graphics/Graphics.hpp>

/*
*THIS IS THE CLIENT PLAY.CPP
*/
Play::Play(char* host, char* port, string name) {
	int sock = server.connect_to_server(host, port);
	if (sock == -1) {
		cout << "Connect error, so quitting" << endl;
		quit_screen(this);
	}
	server.handshake(name);

	// Load Textures
	t_A1		= Graphics::get_texture("A1.png", false, false);		t_B1		= Graphics::get_texture("B1.png", false, false);
	t_A2		= Graphics::get_texture("A2.png", false, false);		t_B2		= Graphics::get_texture("B2.png", false, false);
	t_A_uncon	= Graphics::get_texture("A_uncon.png", false, false);		t_B_uncon	= Graphics::get_texture("B_uncon.png", false, false);
	t_bloomersA	= Graphics::get_texture("bloomersA.png", false, false);		t_bloomersB	= Graphics::get_texture("bloomersB.png", false, false);
	t_bg		= Graphics::get_texture("bg.png", false, false);
	t_dish		= Graphics::get_texture("dish.png", false, false);
	t_dishBroken= Graphics::get_texture("dishbroken.png", false, false);
}

Play::~Play() {
}

void Play::draw() {
	std::vector<int> v = gs.getDrawState();
	if (v.size() != 0) {
		int hp_A = v[0];	int hp_B = v[1];
		int def_A = v[2];	int def_B = v[3];
		int pts_A = v[4];	int pts_B = v[5];
		glEnable(GL_TEXTURE_2D);
		glClearColor(0.0f, 0.0f, 0.0f, 0.0f);
		glClear(GL_COLOR_BUFFER_BIT);
		// draw background
		glBindTexture(GL_TEXTURE_2D, t_bg->obj);
		glBegin(GL_QUADS);
		glTexCoord2f(0.0, 0.0);	glVertex2f(-1.0, -1.0);
		glTexCoord2f(0.0, 1.0);	glVertex2f(-1.0, 1.0);
		glTexCoord2f(1.0, 1.0);	glVertex2f(1.0, 1.0);
		glTexCoord2f(1.0, 0.0);	glVertex2f(1.0, -1.0);
		glEnd();
		glBindTexture(GL_TEXTURE_2D, 0);
		// draw player A
		if (hp_A > 0) {	glBindTexture(GL_TEXTURE_2D, t_A1->obj);
			//std::cout << hp_A << " Normal A texture \n";////////
		} else {	glBindTexture(GL_TEXTURE_2D, t_A_uncon->obj);
			//std::cout << hp_A << " uncons A texture \n";////////
		}
		glBegin(GL_QUADS);
		glTexCoord2f(0.0, 0.0);	glVertex2f(-1.0, -1.0);
		glTexCoord2f(0.0, 1.0);	glVertex2f(-1.0, 1.0);
		glTexCoord2f(1.0, 1.0);	glVertex2f(1.0, 1.0);
		glTexCoord2f(1.0, 0.0);	glVertex2f(1.0, -1.0);
		glEnd();
		// draw player B
		if (hp_B > 0) {	glBindTexture(GL_TEXTURE_2D, t_B1->obj);
			//std::cout << hp_B << " Normal B texture \n";////////
		} else {	glBindTexture(GL_TEXTURE_2D, t_B_uncon->obj);
			//std::cout << hp_B << " uncons B texture \n";////////
		}
		glBegin(GL_QUADS);
		glTexCoord2f(0.0, 0.0);	glVertex2f(-1.0, -1.0);
		glTexCoord2f(0.0, 1.0);	glVertex2f(-1.0, 1.0);
		glTexCoord2f(1.0, 1.0);	glVertex2f(1.0, 1.0);
		glTexCoord2f(1.0, 0.0);	glVertex2f(1.0, -1.0);
		glEnd();
		// draw bloomers
		glBindTexture(GL_TEXTURE_2D, t_bloomersA->obj);
		for (int i = 1; i <= pts_A; i++) { drawBloomer(i);
		}
		glBindTexture(GL_TEXTURE_2D, t_bloomersB->obj);
		for (int i = 1; i <= pts_B; i++) { drawBloomer(6 - i);
		}
		// draw broken dishes
		glBindTexture(GL_TEXTURE_2D, t_dishBroken->obj);
		if (def_A < 0) {
			glBegin(GL_QUADS);
			glTexCoord2f(0.0, 0.0);	glVertex2f(0.0, -1.0);
			glTexCoord2f(0.0, 1.0);	glVertex2f(0.0, 1.0);
			glTexCoord2f(1.0, 1.0);	glVertex2f(-1.0, 1.0);
			glTexCoord2f(1.0, 0.0);	glVertex2f(-1.0, -1.0);
			glEnd();
			def_A++;
		}
		if (def_B < 0) {
			glBegin(GL_QUADS);
			glTexCoord2f(0.0, 0.0);	glVertex2f(0.0, -0.75);
			glTexCoord2f(0.0, 1.0);	glVertex2f(0.0, 0.75);
			glTexCoord2f(1.0, 1.0);	glVertex2f(1.0, 0.75);
			glTexCoord2f(1.0, 0.0);	glVertex2f(1.0, -0.75);
			glEnd();
			def_B++;
		}
		glBindTexture(GL_TEXTURE_2D, 0);
	}
}
void Play::drawBloomer(int b) {
	glBegin(GL_QUADS);
	float bloomersSize = 0.33;	float xOffset;		float yOffset;
	switch (b) {
		case 1:
			xOffset = -0.8;		yOffset = -0.3;
			glTexCoord2f(0.0, 0.0);	glVertex2f(-bloomersSize + xOffset, -bloomersSize + yOffset);
			glTexCoord2f(0.0, 1.0);	glVertex2f(-bloomersSize + xOffset, bloomersSize + yOffset + 0.05);
			glTexCoord2f(1.0, 1.0);	glVertex2f(bloomersSize + xOffset, bloomersSize + yOffset);
			glTexCoord2f(1.0, 0.0);	glVertex2f(bloomersSize + xOffset, -bloomersSize + yOffset);	break;
		case 2:
			xOffset = -0.4;		yOffset = -0.33;
			glTexCoord2f(0.0, 0.0);	glVertex2f(-bloomersSize + xOffset, -bloomersSize + yOffset);
			glTexCoord2f(0.0, 1.0);	glVertex2f(-bloomersSize + xOffset, bloomersSize + yOffset);
			glTexCoord2f(1.0, 1.0);	glVertex2f(bloomersSize + xOffset, bloomersSize + yOffset);
			glTexCoord2f(1.0, 0.0);	glVertex2f(bloomersSize + xOffset, -bloomersSize + yOffset);	break;
		case 3:
			xOffset = 0.0;		yOffset = -0.35;
			glTexCoord2f(0.0, 0.0);	glVertex2f(-bloomersSize + xOffset, -bloomersSize + yOffset);
			glTexCoord2f(0.0, 1.0);	glVertex2f(-bloomersSize + xOffset, bloomersSize + yOffset);
			glTexCoord2f(1.0, 1.0);	glVertex2f(bloomersSize + xOffset, bloomersSize + yOffset);
			glTexCoord2f(1.0, 0.0);	glVertex2f(bloomersSize + xOffset, -bloomersSize + yOffset);	break;
		case 4:
			xOffset = 0.4;	yOffset = -0.33;
			glTexCoord2f(0.0, 0.0);	glVertex2f(-bloomersSize + xOffset, -bloomersSize + yOffset);
			glTexCoord2f(0.0, 1.0);	glVertex2f(-bloomersSize + xOffset, bloomersSize + yOffset);
			glTexCoord2f(1.0, 1.0);	glVertex2f(bloomersSize + xOffset, bloomersSize + yOffset);
			glTexCoord2f(1.0, 0.0);	glVertex2f(bloomersSize + xOffset, -bloomersSize + yOffset);	break;
		case 5:
			xOffset = 0.8;		yOffset = -0.3;
			glTexCoord2f(0.0, 0.0);	glVertex2f(-bloomersSize + xOffset, -bloomersSize + yOffset);
			glTexCoord2f(0.0, 1.0);	glVertex2f(-bloomersSize + xOffset, bloomersSize + yOffset);
			glTexCoord2f(1.0, 1.0);	glVertex2f(bloomersSize + xOffset, bloomersSize + yOffset + 0.02);
			glTexCoord2f(1.0, 0.0);	glVertex2f(bloomersSize + xOffset, -bloomersSize + yOffset);	break;
		default:			break;
	}
	glEnd();
}

void Play::update(float elapsed_time) {
	server.waitLoop(&gs);
}

void Play::handle_event(SDL_Event const &event) {
	//Handle events here:
	if (event.type == SDL_KEYDOWN) {
		if (event.key.keysym.sym == SDLK_ESCAPE) {
			quit_screen(this);
		}
		if (event.key.keysym.sym == SDLK_1) {
			server.sendMsg("A");
		}
		if (event.key.keysym.sym == SDLK_2) {
			server.sendMsg("D");
		}
		if (event.key.keysym.sym == SDLK_3) {
			server.sendMsg("H");
		}
	}
}



