
#include "Server.hpp"

using std::cout;
using std::endl;

using std::vector;
using std::string;


Server::Server() {}

Server::~Server() {}

int Server::connect_to_server(char *host, char *port) {
	//--------------------------------------
	//All this code does is looks up the host and port you passed in and
	//try to connect a socket to it:
	
	//based on example code from:
	//http://people.cs.uchicago.edu/~mark/51081/labs/LAB6/sock.html
	// (which appears to no longer exist... oh well.)

	int newsock = -1;
	struct addrinfo *serv_addrinfo = NULL;
	cout << "Looking up " << host << ":" << port << "..." << endl;
	int ret = getaddrinfo(host, port, NULL, &serv_addrinfo);
	if (ret != 0) {
		cout << "Host lookup error: " << gai_strerror(ret) << endl;
		return -1;
	}
	for (struct addrinfo *current = serv_addrinfo; current != NULL; current = current->ai_next) {
		if (current->ai_socktype != SOCK_STREAM) continue; //no UDP, please.
		cout << "Attempting Connection..."; cout.flush();
		newsock = socket(current->ai_family, current->ai_socktype, current->ai_protocol);
		if (newsock == -1) {
			cout << " (fails on socket)" << endl;
			continue;
		}
		int res = connect(newsock, current->ai_addr, current->ai_addrlen);
		if (res != 0) {
			cout << " (fails on connect)" << endl;
			close(newsock);
			newsock = -1;
			continue;
		}
		cout << " (success)" << endl;
		break;
	}
	freeaddrinfo(serv_addrinfo);

	if (newsock == -1) {
		cout << "No connection worked." << endl;
		return -1;
	}

	sock = newsock;
	return sock;
}

void Server::handshake(string name) {
	string message = (char)name.size() + name;
	message = 'H' + message;
	const char *msg = message.c_str();
	write_buffer.insert(write_buffer.end(), msg, msg + message.size());
}

void Server::sendMsg(string s) {
	string temp;
	temp += 'M';
	temp += (char)s.length();
	temp += s;
	const char *msg = temp.c_str();
	write_buffer.insert(write_buffer.end(), msg, msg + temp.size());
}

int Server::waitLoop(GameState* state) {

	fd_set read_fds;
	fd_set write_fds;
	FD_ZERO(&read_fds);
	FD_SET(sock, &read_fds);
	FD_ZERO(&write_fds);
	if (write_buffer.size()) {
		FD_SET(sock, &write_fds);
	}
	struct timeval* test;
	test = (struct timeval*)malloc(sizeof(struct timeval));
	test->tv_sec = 0;
	test->tv_usec = 1; 
	int ret = select(sock + 1, &read_fds, &write_fds, NULL, test);
	free(test);
	if (ret == -1) {
		perror("Doing select");
	} else if (ret) {
		//If we can write to sock, do a send:
		if (FD_ISSET(sock, &write_fds)) {
			int ret = send(sock, &(write_buffer[0]), write_buffer.size(), 0);
			if (ret == -1) {
				perror("Doing send");
				return 1;
			}
			if (ret > 0) {
				assert(ret <= (signed)write_buffer.size());
				write_buffer.erase(write_buffer.begin(), write_buffer.begin() + ret);
			}
		}
		//if we can read from sock to a recv:
		if (FD_ISSET(sock, &read_fds)) {
			const unsigned int BufferSize = 2000;
			static uint8_t buffer[BufferSize];
			int ret = recv(sock, buffer, BufferSize, 0);
			if (ret == -1) {
				perror("Doing recv");
				return 1;
			}
			read_buffer.insert(read_buffer.end(), buffer, buffer + ret);
			//Now that we have data, we should do something with it:
			while (read_buffer.size() > 2) {
				uint8_t type = read_buffer[0];
				uint32_t size = read_buffer[1];
				if (read_buffer.size() >= 2 + size) {
					if (type == 'T') {
						string message((char *)&(read_buffer[2]), size);
						cout << "Got told: \"" << message << "\"" << endl;
					} else if (type == 'U') {
						assert(size == 6);
						cout << "A: " << (int)read_buffer[2] << " B: " << (int)read_buffer[3] << endl;
						state->hp_A = read_buffer[2]; 
						state->hp_B = read_buffer[3];
						state->def_A = read_buffer[4]; 
						state->def_B = read_buffer[5];
						state->pts_A = read_buffer[6];
						state->pts_B = read_buffer[7];
					} else {
						cout << "Unknown message type '" << (char)type << "' == " << (int)type << "." << endl;
					}

					//clear message out of buffer:
					read_buffer.erase(read_buffer.begin(), read_buffer.begin() + 2 + size);
				}
			}
		}
	} 
	return 0;
}
