#include "Client.hpp"

#include <sys/socket.h>

#include <iostream>
#include <list>
#include <cstdlib>
#include <sstream>
#include <fstream>

using std::cerr;
using std::cout;
using std::endl;
using std::list;
using std::ostringstream;
using std::ifstream;
using std::ofstream;

//vector< Client * > active_clients; //all clients who have open sockets and names.

void client_read_cb(struct ev_loop *loop, ev_io *watcher, int revents) {
	Client *c = (Client *)watcher->data;
	assert(c);
	const unsigned int BufferSize = 2000;
	static uint8_t buffer[BufferSize];
	int ret = recv(watcher->fd, buffer, BufferSize, 0);
	if (ret == -1) {
		cerr << "[" << c->sock << "] Recv error from client." << endl;
		delete c;
		return;
	}
	if (ret == 0) {
		cerr << "[" << c->sock << "] Client is gone." << endl;
		delete c;
		return;
	}
	assert((unsigned)ret <= BufferSize);

	//reset timeout callback:
	ev_timer_again(ev_default_loop(0), &(c->timeout_watcher));

	//process newly read data:
	c->read_buffer.insert(c->read_buffer.end(), buffer, buffer + ret);
	while (c->process_read()) {
		/* keep processing reads until no data left */
		// array with read data?
	}
}

void client_write_cb(struct ev_loop *loop, ev_io *watcher, int revents) {
	Client *c = (Client *)watcher->data;
	assert(c);

	//If we don't have anything to write, stop firing this callback:
	if (c->write_buffer.empty()) {
		ev_io_stop(loop, watcher);
		//also, if client should close after sending data, close!
		if (c->suicide) {
			delete c;
		}
		return;
	}
	int ret = send(watcher->fd, &(c->write_buffer[0]), c->write_buffer.size(), 0);
	if (ret == -1) {
		cerr << "[" << c->sock << "] Send error to client." << endl;
		delete c;
		return;
	}
	if (ret > 0) {
		assert(ret <= (signed)c->write_buffer.size());
		c->write_buffer.erase(c->write_buffer.begin(), c->write_buffer.begin() + ret);
		//reset timeout callback:
		ev_timer_again(ev_default_loop(0), &(c->timeout_watcher));
	}
}

void client_timeout_cb(struct ev_loop *loop, ev_timer *watcher, int revents) {
	Client *c = (Client *)watcher->data;
	assert(c);
	if (c->suicide) {
		delete c;
		return;
	}
	cerr << "[" << c->sock << "] Suiciding a timed-out client." << endl;
	c->send_error("You timed out.");
	ev_timer_again(ev_default_loop(0), &(c->timeout_watcher));
}

bool check_file(string name) {
	ifstream file(("results/" + name).c_str());
	char temp = 0;
	if (file >> temp) return true;
	return false;
}

void create_file(string name) {
	if (!check_file(name)) {
		ofstream file(("results/" + name).c_str());
		file << "1";
		file.close();
	}
	assert(check_file(name));
}


Client::Client(int _sock, GameState* _state) : sock(_sock), state(_state), suicide(false) {

	ev_io_init(&read_watcher, client_read_cb, sock, EV_READ);
	ev_io_init(&write_watcher, client_write_cb, sock, EV_WRITE);
	ev_timer_init(&timeout_watcher, client_timeout_cb, 0., 10000.);
	read_watcher.data = this;
	write_watcher.data = this;
	timeout_watcher.data = this;
	
	ev_io_start(ev_default_loop(0), &read_watcher);
	ev_io_start(ev_default_loop(0), &write_watcher);
	ev_timer_again(ev_default_loop(0), &timeout_watcher);
}

// send signal?
Client::~Client() {

	cout << "[" << sock << "] Disconnecting." << endl;

	ev_io_stop(ev_default_loop(0), &read_watcher);
	ev_io_stop(ev_default_loop(0), &write_watcher);
	ev_timer_stop(ev_default_loop(0), &timeout_watcher);

	//If we are listed as an active client, remove ourselves:
	if (state->player1 == this) {
		state->player2->send_error("Someone dropped out! :(");
	}
	if (state->player2 == this) {
		state->player1->send_error("Someone dropped out! :(");
	}
	state->reinit();

	if (sock == -1) {
		cerr << "[" << sock << "] Trying to shut down with already-closed socket" << endl;
		return;
	} else {
		int ret = close(sock);
		if (ret == -1) {
			cerr << "[" << sock << "] Error closing socket." << endl;
		}
		sock = -1;
	}
}

void Client::send_message(string message) {
	cout << "[" << name << "] sending message '" << message << "'" << endl;
	message = (char)message.size() + message;
	message = 'T' + message;
	write_data(message.c_str(), message.size());
}

void Client::send_update() {
	string message; 
	message += 'U';
	message += (char)6;
	cout << "A:" << state->hp_A << " B: " << state->hp_B << endl;
	message = message + (char)state->hp_A + (char)state->hp_B + (char)state->def_A + 
		(char)state->def_B + (char)state->pts_A + (char)state->pts_B;
	write_data(message.c_str(), message.size());
}

void Client::send_error(string message) {
	if (suicide == true) {
		cerr << "[" << sock << "] Double-suicide!" << endl;
	}
	cout << "[" << sock << "] Suicide via error '" << message << "'" << endl;
	message = (char)message.size() + message;
	message = 'T' + message;
	write_data(message.c_str(), message.size());
	suicide = true;
}

void Client::write_data(const void *data, unsigned int length) {
	if (sock == -1) {
		cerr << "Trying to write data to a closed socket" << endl;
		return;
	}
	write_buffer.insert(write_buffer.end(), (uint8_t *)data, (uint8_t *)data + length);
	//start write_watcher if not already started:
	ev_io_start(ev_default_loop(0), &write_watcher);
}

bool Client::process_read() {
	//do we have enough for a header?
	if (read_buffer.size() < 2) return false;
	uint8_t type = read_buffer[0];
	uint32_t size = read_buffer[1];
	//do we have this whole message?
	if (size + 2U > read_buffer.size()) return false;
	if (name == "") {
		//before handshake, want 'H' == Handshake message.
		if (type == 'H') {
			if (size > 0) {
				name = string((char *)&(read_buffer[2]), (char *)&(read_buffer[2]) + size);
				for (unsigned int i = 0; i < name.size(); ++i) {
					if (!isalnum('a')) {
						send_error("Alphanumeric names only, please.");
						return false;
					}
				}
				cout << "[" << sock << "] Got player '" << name << "'" << endl;
				fflush(stdout);
				
				if (!state->player1) {
					send_message("Welcome! The game will begin shortly...");
					state->player1 = this;
					if (state->player2) {
						state->game_on = true;
						cout << "MAMMA MIA!" << endl;
					}
				}
				else if (!state->player2) {
					send_message("Welcome! The game will begin shortly...");
					state->player2 = this;
					if (state->player1) {
						state->game_on = true;
						cout << "MAMMA MIA!" << endl;
					}
				}
				else {
					send_error("Sorry, no space in the current game.");
				}
			} else {
				send_error("Zero-length handshake");
			}
		} else {
			send_error("Expecting handshake");
			cout << "[" << sock << "] Suicide on unknown message '" << (char)type << "' == " << (int)type << "." << endl;
		}
	} else {
		//after handshake, want 'M' == Move message.
		if (type == 'M') {
			if (state->get_turn(this) && state->game_on) {
				cout << name << "'s msg: (" << size << ") ";
				for (unsigned i=2; i<read_buffer.size(); i++) {
					cout << read_buffer[i];
				}
				cout << endl;
				//if (size == 1) {
					char move = read_buffer[2];
					// Attack
					if (move == 'A') {
						//cout << name << " attacked!" << endl; 
						state->attack(this);
					} 
					// Defend
					else if (move == 'D') {
						//cout << name << " defended!" << endl; 
						state->defend(this);
					} 
					// Hang shirt
					else if (move == 'H') {
						//cout << name << " hung a shirt!" << endl; 
						state->hang(this);
					}
					else {
						send_message("Unknown move, try again.");
					}
				//} 
				//else {
					//send_error("Expecting single-byte move");
				//}
			}
			state->player1->send_update();
			state->player2->send_update();
		} 
		else {
			send_error("Expecting move");
			cout << "[" << sock << "] Suicide on unknown message '" << (char)type << "' == " << (int)type << "." << endl;
		}
	}
	//Erase message:
	read_buffer.erase(read_buffer.begin(), read_buffer.begin() + 2 + size);
	return true;
}
